import uuid

import msal
import requests
from django.utils.crypto import get_random_string

from .exceptions import DomainException, WrongTokenException
from .settings import api_settings
from rest_framework_jwt.settings import api_settings as jwt_api_settings


def get_msal_public_app():
    """
    Get MSAL Public Application
    :return:
    """
    msal_client_id = api_settings.MSAL_CLIENT_ID
    msal_authority_url = api_settings.MSAL_AUTHORITY_URL

    assert (msal_client_id is not None)

    return msal.PublicClientApplication(
        msal_client_id,
        authority=msal_authority_url
    )


def get_msal_confidential_app():
    """
    Get MSAL Confidential Application
    :return:
    """
    msal_client_id = api_settings.MSAL_CLIENT_ID
    msal_client_secret = api_settings.MSAL_CLIENT_SECRET
    msal_authority_url = api_settings.MSAL_AUTHORITY_URL

    assert (msal_client_id is not None)
    assert (msal_client_secret is not None)

    return msal.ConfidentialClientApplication(
        msal_client_id,
        client_credential=msal_client_secret,
        authority=msal_authority_url
    )


def build_auth_url(scopes=None, state=None):
    """
    Generate Auth URL

    :param scopes: Permission the client request from the user
    :param state: State string to verify the integrity of the response
    :return:
     - string: a url for user to authenticate
    """
    msal_redirect_url = api_settings.MSAL_REDIRECT_URL
    msal_scopes = api_settings.MSAL_SCOPES

    return get_msal_public_app().get_authorization_request_url(
        scopes or msal_scopes,
        state=state or str(uuid.uuid4()),
        redirect_uri=msal_redirect_url
    )


def get_user_jwt_token(code, scopes=None):
    msal_redirect_url = api_settings.MSAL_REDIRECT_URL
    msal_scopes = api_settings.MSAL_SCOPES
    msal_allow_domains = api_settings.MSAL_ALLOW_DOMAINS

    User = api_settings.MSAL_USER_HANDLER
    tokens = get_msal_confidential_app().acquire_token_by_authorization_code(
        code,
        scopes or msal_scopes,
        redirect_uri=msal_redirect_url
    )

    try:
        microsoft_info = get_microsoft_info(tokens['access_token'])
    except KeyError:
        raise WrongTokenException()

    # check allow domains
    if not '*' in msal_allow_domains:
        if microsoft_info['mail'].split('@')[1] not in msal_allow_domains:
            raise DomainException()

    user = get_user_by_email(microsoft_info['mail'])
    if not user:
        # User not found in the system, we should create a new user
        user = User.objects.create_user(username=tokens['id_token_claims']['preferred_username'],
                                        email=microsoft_info['mail'],
                                        password=get_random_string(length=12),
                                        first_name=microsoft_info.get('givenName', ''),
                                        last_name=microsoft_info.get('surname', ''))
    jwt_payload_handler = jwt_api_settings.JWT_PAYLOAD_HANDLER
    jwt_encode_handler = jwt_api_settings.JWT_ENCODE_HANDLER
    payload = jwt_payload_handler(user)
    token = jwt_encode_handler(payload)
    return token


def get_microsoft_info(access_token):
    r = requests.get(r'https://graph.microsoft.com/v1.0/me/', headers={'Authorization': f'Bearer {access_token}'})
    return r.json()


def get_user_by_email(email):
    User = api_settings.MSAL_USER_HANDLER
    user = User.objects.filter(is_active=True, email=email)
    if not user:
        return None

    return user[0]



from rest_framework.exceptions import APIException


class CodeException(APIException):
    status_code = 401
    default_detail = 'Invalid authorization code'
    default_code = 'code_error'


class DomainException(APIException):
    status_code = 403
    default_detail = "The account from the domain you sign in are not allowed"
    default_code = 'invalid_domain_error'


class StateException(APIException):
    status_code = 401
    default_detail = 'Invalid state, please try again'
    default_code = 'state_error'


class WrongTokenException(APIException):
    status_code = 401
    default_detail = 'The access token is no longer valid, please try logging in again'
    default_code = 'access_token_error'



# -*- coding: utf-8 -*-
from django.conf.urls import url

from .views import MSALLoginView, MSALLoginWithCodeView

app_name = 'drf_msal_jwt'
urlpatterns = [
    url(r'msal/login_url', MSALLoginView.as_view()),
    url(r'msal/login_with_code', MSALLoginWithCodeView.as_view())
    ]



import uuid

from rest_framework.response import Response
from rest_framework.views import APIView

from drf_msal_jwt.exceptions import StateException
from drf_msal_jwt.serializer import CodeSerializer
from drf_msal_jwt.settings import api_settings
from drf_msal_jwt.utils import build_auth_url, get_user_jwt_token


class MSALLoginView(APIView):

    authentication_classes = []
    permission_classes = []

    def get(self, request, format=None):
        # Create session by uuid
        state = str(uuid.uuid4())
        request.session['msal_state'] = state

        return Response({
            'login_url': build_auth_url(state=state)
        })


class MSALLoginWithCodeView(APIView):

    authentication_classes = []
    permission_classes = []
    msal_check_state = api_settings.MSAL_CHECK_STATE

    def post(self, request, format=None):
        code_serialized = CodeSerializer(request.data)

        if msal_check_state and request.session.get('msal_state', '') != code_serialized.data['state']:
            raise StateException()

        user_token = get_user_jwt_token(code_serialized.data['code'])
        return Response({
            'token': user_token
        })





from rest_framework import serializers


class CodeSerializer(serializers.Serializer):
    """
    Serializer class for serialize code from the user
    """
    code = serializers.CharField(max_length=2000)
    state = serializers.CharField(max_length=36, allow_blank=True, allow_null=True)



from django.conf import settings
from rest_framework.settings import APISettings

USER_SETTINGS = getattr(settings, 'DRF_MSAL', None)

DEFAULTS = {
    'MSAL_CLIENT_ID': None,
    'MSAL_CLIENT_SECRET': None,
    'MSAL_AUTHORITY_URL': 'https://login.microsoftonline.com/common/',
    'MSAL_REDIRECT_URL': None,
    'MSAL_SCOPES': ["User.ReadBasic.All"],
    'MSAL_USER_HANDLER': 'django.contrib.auth.models.User',
    'MSAL_ALLOW_DOMAINS': ['*'],
    'MSAL_CHECK_STATE': True
}

IMPORT_STRINGS = (
    'MSAL_USER_HANDLER'
)

api_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)
